﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using System.IO.IsolatedStorage;
using System.IO;
using System.Xml.Serialization;
using System.Xml;
using System.Diagnostics;

namespace CS185_HW2
{
    /// <summary>
    /// CS185 Homework 3 
    /// SJSU Students
    /// 10/6/2010
    /// This application sends morse code signals as light flashes for the given message typed by the user.
    /// </summary>
    public partial class MainPage : PhoneApplicationPage
    {
        
        // Constructor
        public MainPage()
        {
            InitializeComponent();
            mMorsePanel.MessageSendComplete += new EventHandler(mMorsePanel_MessageSendComplete);
        }

        /// <summary>
        /// When the message is complete, we go back to half screen view.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void mMorsePanel_MessageSendComplete(object sender, EventArgs e)
        {
            pnlControls.Visibility = System.Windows.Visibility.Visible;
        }

        /// <summary>
        /// The send button is clicked, lets send a message
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSend_Click(object sender, RoutedEventArgs e)
        {
            actionSend();
            
        }

        /// <summary>
        /// internal helper function which logs the messaage sent and sends the message.
        /// </summary>
        private void actionSend()
        {
            Settings.Current.LastMessages.Insert(0, tbMessage.Text);
            if (Settings.Current.LastMessages.Count > 3)
            {
                Settings.Current.LastMessages.RemoveRange(3,Settings.Current.LastMessages.Count - 3);
            }
            SetButtons();
            SendMessage(tbMessage.Text);
        }

        /// <summary>
        /// Actually send the message!
        /// </summary>
        /// <param name="msg"></param>
        private void SendMessage(string msg)
        {
            if (cbFullScreenMode.IsChecked == true)
            {
                pnlControls.Visibility = System.Windows.Visibility.Collapsed;
            }
            this.mMorsePanel.SendMessage(msg);
        }

        /// <summary>
        /// The method called when the application page loads
        /// Here we will load the preview settings and set our controls.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PhoneApplicationPage_Loaded(object sender, RoutedEventArgs e)
        {
            SetButtons();
            sldrIntensity.Value = Settings.Current.SliderValue;
            Color tmpCol = GetColorForIntensity(Settings.Current.SliderValue);
            mMorsePanel.LightColor = tmpCol;
            if (Settings.Current.LastView == ViewSettings.FullScreen)
            {
                cbFullScreenMode.IsChecked = true;
            }
            else
            {
                cbFullScreenMode.IsChecked = false;
            }

        }

        /// <summary>
        /// helper function to set the previously sent message buttons
        /// </summary>
        private void SetButtons()
        {
            if (Settings.Current.LastMessages.Count >= 3)
            {
                btnMessage1.Visibility = System.Windows.Visibility.Visible;
                btnMessage2.Visibility = System.Windows.Visibility.Visible;
                btnMessage3.Visibility = System.Windows.Visibility.Visible;
                btnMessage1.Content = Settings.Current.LastMessages[0];
                btnMessage2.Content = Settings.Current.LastMessages[1];
                btnMessage3.Content = Settings.Current.LastMessages[2];
            }
            else if (Settings.Current.LastMessages.Count == 2)
            {
                btnMessage1.Visibility = System.Windows.Visibility.Visible;
                btnMessage2.Visibility = System.Windows.Visibility.Visible;
                btnMessage1.Content = Settings.Current.LastMessages[0];
                btnMessage2.Content = Settings.Current.LastMessages[1];
            }
            else if (Settings.Current.LastMessages.Count == 1)
            {
                btnMessage1.Visibility = System.Windows.Visibility.Visible;
                btnMessage1.Content = Settings.Current.LastMessages[0];
            }
        }

        /// <summary>
        /// we want to send a message!
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SendMessage_Click(object sender, RoutedEventArgs e)
        {
            
            SendMessage((String)((Button) sender).Content);
        }

        /// <summary>
        /// The user checked the button to show fullscreen messages.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cbFullScreenMode_Click(object sender, RoutedEventArgs e)
        {
            if (((CheckBox)sender).IsChecked == true)
            {
                Settings.Current.LastView = ViewSettings.FullScreen;
            }
            else
            {
                Settings.Current.LastView = ViewSettings.HalfScreen;
            }
            
        }



        byte MIN = 0;
        byte MAX = 255;
        /// <summary>
        /// Handel slider changes for intensity.  When this is updated we will get the new color, set the morse code panel with it and save the color in the settings
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void sldrIntensity_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            Settings.Current.SliderValue = ((byte)e.NewValue);
            Color tmpColor = GetColorForIntensity(((byte)e.NewValue));

            mMorsePanel.backBrush.GradientStops[0].Color = tmpColor;
            ColorAnimation mdit1 = mMorsePanel.MorseDit.Children[0] as ColorAnimation;
            ColorAnimation mdit2 = mMorsePanel.MorseDit.Children[1] as ColorAnimation;
            ColorAnimation mdash1 = mMorsePanel.MorseDash.Children[0] as ColorAnimation;
            ColorAnimation mdash2 = mMorsePanel.MorseDash.Children[1] as ColorAnimation;

            mdit1.To = tmpColor;
            mdit2.To = Color.FromArgb(((byte)(e.NewValue / 2)), tmpColor.R, tmpColor.G, tmpColor.B);
            mdash1.To = tmpColor;
            mdash2.To = Color.FromArgb(((byte)(e.NewValue / 2)), tmpColor.R, tmpColor.G, tmpColor.B);
        }

        /// <summary>
        /// helper function to get color for the intensity.
        /// </summary>
        /// <param name="intensity"></param>
        /// <returns></returns>
        private Color GetColorForIntensity(byte intensity)
        {
            Color tmpColor = mMorsePanel.LightColor;
            tmpColor = Color.FromArgb(intensity, tmpColor.R, tmpColor.G, tmpColor.B);
            return tmpColor;
        }

        private Color getColor(int value)
        {
            byte mod = (byte)((int)value % MAX);
            byte diff = (byte)(MAX - mod);
            byte alpha = 255;
            switch ((int)value / MAX)
            {
                case 0: return Color.FromArgb(alpha, MAX, mod, MIN);
                case 1: return Color.FromArgb(alpha, diff, MAX, MIN);
                case 2: return Color.FromArgb(alpha, MIN, MAX, mod);
                case 3: return Color.FromArgb(alpha, MIN, diff, MAX);
                case 4: return Color.FromArgb(alpha, mod, MIN, MAX);
                case 5: return Color.FromArgb(alpha, MAX, MIN, diff);
                default: return Colors.Black;
            }
        }
        /// <summary>
        /// when the enter key is hit, we will go ahead and lose focus on the textbox to hide the soft keyboard.
        /// Also send the current message.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tbMessage_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Enter)
            {
                FocusHelper.Focus(this);
                actionSend();
            }
        }
     
       

        


    }
}
