﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace CS185_HW2
{
    /// <summary>
    /// This class encapsulates the morse code logic and displays the messages as pulses of light (using storyboards and animations.)
    /// </summary>
    public partial class MorsePanel : UserControl
    {
        private Queue< MorseCodeElements> _message;
        
        public MorsePanel()
        {
            InitializeComponent();
            _message = new Queue<MorseCodeElements>();
//            LightColor = Color.FromArgb(0xFF, 0xC7, 0xD6, 0x3D);
            LightColor = Color.FromArgb(0xFF, 0xFF, 0xFF, 0xFF);
        }


        public event EventHandler MessageSendComplete;

        /// <summary>
        /// Takes the next message part from the message queue and plays the associated storyboard.
        /// </summary>
        protected  void PlayNext()
        {
             MorseCodeElements msg = MorseCodeElements.None ;
            lock (_message )
            {
                if (_message.Count > 0)
                {
                    msg = _message.Dequeue();
                }
                else
                {
                    MessageSendComplete(this, new EventArgs());
                }
                
            }
           
            switch (msg)
            {
                case MorseCodeElements.Dash:
                    this.MorseDash.Begin();
                    break;
                case MorseCodeElements.Dit:
                    this.MorseDit.Begin();
                    break;
                case MorseCodeElements.IntraChar:
                    this.morseIntraCharacter.Begin();
                    break;
                case MorseCodeElements.LongSpace:
                    this.morseLongGap.Begin();
                    break;
                case MorseCodeElements.ShortSpace:
                    this.morseShortGap.Begin();
                    break;
                default:
                    break;

            }
        }


        /// <summary>
        /// send a message!
        /// </summary>
        /// <param name="message"></param>
        public void SendMessage(string message)
        {

            enqueueMessage(message);
            PlayNext();
            
        }

        /// <summary>
        /// Enqueue the next message into a message queue
        /// </summary>
        /// <param name="message"></param>
        private void enqueueMessage(string message)
        {
            lock (_message)
            {
                 _message.Clear();
          
            foreach (char c in message.ToLower().ToCharArray())
            {
                switch (c)
                {
                    case 'a':
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        break;
                    case 'b':
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        break;
                    case 'c':
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        break;
                    case 'd':
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        break;
                    case 'e':
                        _message.Enqueue(MorseCodeElements.Dit);
                        break;
                    case 'f':
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        break;
                    case 'g':
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        break;
                    case 'h':
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        break;
                    case 'i':
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        break;
                    case 'j':
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        break;
                    case 'k':
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        break;
                    case 'l':
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        break;
                    case 'm':
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        break;
                    case 'n':
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        break;
                    case 'o':
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        break;
                    case 'p':
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        break;
                    case 'q':
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        break;
                    case 'r':
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        break;
                    case 's':
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        break;
                    case 't':
                        _message.Enqueue(MorseCodeElements.Dash);
                        break;
                    case 'u':
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        break;
                    case 'v':
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);

                        break;
                    case 'w':
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        break;
                    case 'x':
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        break;
                    case 'y':
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        break;
                    case 'z':
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        break;
                    case '1':
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        break;
                    case '2':
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        break;
                    case '3':
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        break;
                    case '4':
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        break;
                    case '5':
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        break;
                    case '6':
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        break;
                    case '7':
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        break;
                    case '8':
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        break;
                    case '9':
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dit);
                        break;
                    case '0':
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        _message.Enqueue(MorseCodeElements.IntraChar);
                        _message.Enqueue(MorseCodeElements.Dash);
                        break;
                    case ' ':
                        _message.Enqueue(MorseCodeElements.ShortSpace);
                        _message.Enqueue(MorseCodeElements.IntraChar);

                        break;
                    default:
                        break;

                }
                _message.Enqueue(MorseCodeElements.ShortSpace);
            }
            }
           
        }

        /// <summary>
        /// as each storyboard completes, go and look at the next message.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Morse_Completed(object sender, EventArgs e)
        {
            PlayNext();
        }
        
        /// <summary>
        /// The light color used when flashing the light.
        /// </summary>
        public Color LightColor { get; set; }
       
      

    }
}
