﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.Generic;
using System.IO.IsolatedStorage;
using System.IO;
using System.Xml.Serialization;
using System.Xml;
using System.Diagnostics;

namespace CS185_HW2
{
    /// <summary>
    /// Our different view settings
    /// </summary>
    public enum ViewSettings
    {
        FullScreen,
        HalfScreen
    }

    /// <summary>
    /// Settings class which will be serialized into our local storage and used throught the app.
    /// </summary>
    public class Settings
    {
        public List<String> LastMessages { get; set; }
        public byte SliderValue { get; set; }
        public ViewSettings LastView { get; set; }
        private static Settings _Settings = null;

        private static string _fileName = "settings.xml";

        /// <summary>
        /// Helper function which deserializes the settings from local storage.
        /// </summary>
        /// <returns></returns>
        private static  Settings LoadSettings()
        {
            try
            {
                IsolatedStorageFile isf = IsolatedStorageFile.GetUserStoreForApplication();

                if (isf.FileExists(_fileName))
                {
                    Settings retSettings = null;
                    using (IsolatedStorageFileStream fs = isf.OpenFile("settings.xml", System.IO.FileMode.Open, System.IO.FileAccess.Read))
                    {
                        XmlSerializer serializer = new XmlSerializer(typeof(Settings));
                        retSettings = (Settings)serializer.Deserialize(fs);
                    }

                    return retSettings;
                }
                else
                {
                    Settings set = new Settings();
                    set.LastMessages = new List<String>();
                    set.LastView = ViewSettings.FullScreen;
                    set.SliderValue = 255;
                    return set;
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine("Problem reading the settings file. EX: " + ex.ToString());
                Settings set = new Settings();
                set.LastMessages = new List<String>();
                set.LastView = ViewSettings.FullScreen;
                set.SliderValue = 255;
                return set;
            }

        }

        /// <summary>
        /// Saves (serializes ) the settings to the local storage folder
        /// </summary>
        /// <param name="settings"></param>
        public static void Save(Settings settings)
        {
            try
            {
                IsolatedStorageFile isf = IsolatedStorageFile.GetUserStoreForApplication();
                using (IsolatedStorageFileStream fs = isf.OpenFile("settings.xml", System.IO.FileMode.Create, System.IO.FileAccess.ReadWrite))
                {
                    XmlSerializer serializer = new XmlSerializer(typeof(Settings));
                    serializer.Serialize(fs, settings);
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine("Problem writing the settings file. EX: " + ex.ToString());
            }
        }


        /// <summary>
        /// The current settings used throughout the app.
        /// </summary>
        public static Settings Current
        {
            get
            {
                if (_Settings == null)
                {
                    _Settings = LoadSettings();
                    return _Settings;
                }
                else
                {
                    return _Settings;
                }
            }

        }
    }
}
